#include "shadow_map.h"

#include "Logger.h"

shadow_map::shadow_map() :
    depth_texture_(nullptr),
    depth_stencil_view_(nullptr),
    shader_resource_view_(nullptr)
{
    ZeroMemory(&viewport_, sizeof(viewport_));
}

shadow_map::~shadow_map()
{
    shutdown();
}

bool shadow_map::initialize(ID3D11Device* device, int shadow_map_width, int shadow_map_height)
{
    D3D11_TEXTURE2D_DESC text_desc = {};
    text_desc.Width = shadow_map_width;
    text_desc.Height = shadow_map_height;
    text_desc.MipLevels = 1;
    text_desc.ArraySize = 1;
    text_desc.Format = DXGI_FORMAT_R32_TYPELESS;
    text_desc.SampleDesc.Count = 1;
    text_desc.Usage = D3D11_USAGE_DEFAULT;
    text_desc.BindFlags = D3D11_BIND_DEPTH_STENCIL | D3D11_BIND_SHADER_RESOURCE;

    if (FAILED(device->CreateTexture2D(&text_desc, nullptr, &depth_texture_)))
    {
        Logger::Get().Log("Failed to create depth texture", __FILE__, __LINE__, Logger::LogLevel::Error);
        return false;
    }

    D3D11_DEPTH_STENCIL_VIEW_DESC dsv_Desc = {};
    dsv_Desc.Format = DXGI_FORMAT_D32_FLOAT;
    dsv_Desc.ViewDimension = D3D11_DSV_DIMENSION_TEXTURE2D;
    dsv_Desc.Texture2D.MipSlice = 0;

    if (FAILED(device->CreateDepthStencilView(depth_texture_, &dsv_Desc, &depth_stencil_view_)))
    {
        Logger::Get().Log("Failed to create depth texture", __FILE__, __LINE__, Logger::LogLevel::Error);
        return false;
    }

    D3D11_SHADER_RESOURCE_VIEW_DESC srv_desc = {};
    srv_desc.Format = DXGI_FORMAT_R32_FLOAT;
    srv_desc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2D;
    srv_desc.Texture2D.MostDetailedMip = 0;
    srv_desc.Texture2D.MipLevels = 1;

    if (FAILED(device->CreateShaderResourceView(depth_texture_, &srv_desc, &shader_resource_view_)))
    {
        Logger::Get().Log("Failed to create shader resource view", __FILE__, __LINE__, Logger::LogLevel::Error);
        return false;
    }

    viewport_.Width = static_cast<FLOAT>(shadow_map_width);
    viewport_.Height = static_cast<FLOAT>(shadow_map_height);
    viewport_.MinDepth = 0.0f;
    viewport_.MaxDepth = 1.0f;
    viewport_.TopLeftX = 0.0f;
    viewport_.TopLeftY = 0.0f;

    Logger::Get().Log("Shadow map initialized", __FILE__, __LINE__, Logger::LogLevel::Info);

    return true;
        
}

void shadow_map::set_render_target(ID3D11DeviceContext* context)
{
    context->OMSetRenderTargets(0, nullptr, depth_stencil_view_);
    context->RSSetViewports(1, &viewport_);
}

void shadow_map::clear_render_target(ID3D11DeviceContext* context, float depth)
{
    context->ClearDepthStencilView(depth_stencil_view_, D3D11_CLEAR_DEPTH, depth, 0);
}

ID3D11ShaderResourceView* shadow_map::get_shader_resource_view() const
{
    return shader_resource_view_;
}

void shadow_map::shutdown()
{
    
    if (shader_resource_view_) {
        shader_resource_view_->Release();
        shader_resource_view_ = nullptr;
    }
    if (depth_stencil_view_) {
        depth_stencil_view_->Release();
        depth_stencil_view_ = nullptr;
    }
    if (depth_texture_) {
        depth_texture_->Release();
        depth_texture_ = nullptr;
    }
    
    Logger::Get().Log("Shadow map shut down", __FILE__, __LINE__, Logger::LogLevel::Shutdown);
}